#include "pdfqcd03.h"

const double lam4nlo=0.326;
const double lam5nlo=0.226;
const double lam4lo=0.192;
const double lam5lo=0.146;
const double lam3ffn=0.395;
const double lam4ffn=0.309;
// Cteq6 LO lam's differ from CTEQ5:
const double lamC64lo=0.215;
const double lamC65lo=0.165;
// I've shifted mrst lam4's so that they match at 4.5 GeV
// with the lam5's, giving the correct alpha_s(MZ).
const double lam4mrst=0.318;
const double lam4mrst4=0.244;
const double lam4mrst5=0.405;
const double lam4mrst6=0.3227;
const double lam4mrst7=0.3115;
const double lam5mrst=0.220;
const double lam5mrst4=0.164;
const double lam5mrst5=0.289;
const double lam5mrst6=0.2237;
const double lam5mrst7=0.215;
const double mb=4.5;
double lam4;
double lam;
double b0;
double b1;

extern "C" {

void setctq5_(long int* iset);
double ctq5pdf_(long int* iparton, double* x, double* Q);
void setctq6_(long int* iset);
double ctq6pdf_(long int* iparton, double* x, double* Q);
}

long int iset;

pdf_type pdft;

c_mrst parton;

void pdfinit(pdf_type pdftt, long int iiset) 
{
  double nfl;
  iset=iiset;
  pdft=pdftt;
  if (pdft==cteq5) {
     setctq5_(&iset);
     if (iset==6) {
       nfl=3.0;
       lam=lam3ffn;
     } else if (iset==7) {
       nfl=4.0;
       lam=lam4ffn;
     } else if (iset==3) {
       nfl=5.0;
       lam=lam5lo;
       lam4=lam4lo;
     } else {
       nfl=5.0;
       lam=lam5nlo;
       lam4=lam4nlo;
     }
  } else if (pdft==cteq6) {
     setctq6_(&iset);
     if (iset==4) {
       nfl=5.0;
       lam=lamC65lo;
       lam4=lamC64lo;
     } else {
       nfl=5.0;
       lam=lam5nlo;
       lam4=lam4nlo;
     }
  } else {
     if (iset==4) {
       nfl=5.0;
       lam=lam5mrst4;
       lam4=lam4mrst4;
     } else if (iset==5) {
       nfl=5.0;
       lam=lam5mrst5;
       lam4=lam4mrst5;
     } else if (iset==6) {
       nfl=5.0;
       lam=lam5mrst6;
       lam4=lam4mrst6;
     } else if (iset==7) {
       nfl=5.0;
       lam=lam5mrst7;
       lam4=lam4mrst7;
     } else {
       nfl=5.0;
       lam=lam5mrst;
       lam4=lam4mrst;
     }
  }
  b0=(11.0-2.0/3.0*nfl);
  b1=102.0-38.0/3.0*nfl;
}

double pdf(long int iparton, double x, double Q)
{
  if (pdft==mrst) {
    parton.mrst99(x,Q,iset);
    if (iparton==0) return parton.cont.glu/x;
    if (iparton==1) return (parton.cont.upv+parton.cont.usea)/x;
    if (iparton==2) return (parton.cont.dnv+parton.cont.dsea)/x;
    if (iparton==-1) return (parton.cont.usea)/x;
    if (iparton==-2) return (parton.cont.dsea)/x;
    if (iparton==3||iparton==-3) return parton.cont.str/x;
    if (iparton==4||iparton==-4) return parton.cont.chm/x;
    if (iparton==5||iparton==-5) return parton.cont.bot/x;
  } else if (pdft==cteq5) {
    return ctq5pdf_(&iparton,&x,&Q);
  } else {
    return ctq6pdf_(&iparton,&x,&Q);
  }
}


double pdfeff(double x, double Q)
{
    double af=9.0/4.0;
    double pd=0.0;
    for (long int iparton=-5;iparton<6;iparton++) {
        if (iparton==0) {pd+= af*pdf(iparton,x,Q);
        } else {pd+=pdf(iparton,x,Q);}
    }
    return pd/af;
}

void pdfpairs(collider col, double x1, double x2, double Q, double &fgg, 
    double &fgq, double &fqg, double &fqiqj, double &fqiqi, double &fqiai)
{
    long int iparton, aparton;
    double fq1, fq2, fa1, fa2, fg1, fg2;
    double fqt1=0.0;
    double fqt2=0.0;
    fqiqi=0.0;
    fqiai=0.0;
    for (iparton=1;iparton<6;iparton++) {
        aparton=-iparton;
        fq1=pdf(iparton,x1,Q);
        fa1=pdf(aparton,x1,Q);
        if (col==pp) {
           fa2=pdf(aparton,x2,Q);  // hadron 2 is p.
           fq2=pdf(iparton,x2,Q);  // hadron 2 is p.
        } else {
           fa2=pdf(iparton,x2,Q);  // hadron 2 is pbar.
           fq2=pdf(aparton,x2,Q);  // hadron 2 is pbar.
        }
    fqt1+=fq1+fa1;
    fqt2+=fq2+fa2;
    fqiqi+=fq1*fq2+fa1*fa2;
    fqiai+=fq1*fa2+fa1*fq2;
    }
    fqiqj=fqt1*fqt2 - fqiqi - fqiai;
    iparton=0;
    fg1=pdf(iparton,x1,Q);
    fg2=pdf(iparton,x2,Q);
    fgg = fg1*fg2;
    fgq = fg1*fqt2;
    fqg = fqt1*fg2;
}

double pdfquarks(double x, double Q)
{
    long int iparton, aparton;
    double fq=0.0;
    for (iparton=1;iparton<6;iparton++) {
        aparton=-iparton;
        fq+= pdf(iparton,x,Q)
            +pdf(aparton,x,Q);
    }
    return fq;
}


double alphas(double mu)
{
  double AA, nf, ll, aln, bb0, bb1;
  ll=lam;
  bb0=b0;
  bb1=b1;
  if ( ((pdft==mrst)||((iset!=6)&&(iset!=7))) && (mu<=mb)) {
    bb0+=2.0/3.0;
    bb1+=38.0/3.0;
    ll=lam4;
  } 
  aln=log(mu*mu/ll/ll);
  if ( aln<=0.0  ) {  
       cerr<<" Value of Q given was "<<mu<<"\n";
       cerr<<" Value of iset was "<<iset<<"\n";
       cerr<<" You have reached the Landau singularity.\n";
       cerr<<" All hope is lost!\n";
       exit(1);
  }
  AA=4.0*PI/bb0/aln;
  if ((pdft==cteq5&&iset==3)||(pdft==cteq6&&iset==4)) {
    return AA;
  } else {
    return AA*(1.0-bb1*log(aln)/aln/bb0/bb0);
  }
}

const double alphasMZ = 0.1273;
double alphasLO(double QQ) {
  double AA=1.0/alphasMZ + (23.0/6.0/PI)*log(QQ/MZ);
  if ( QQ<=mb ) {AA = AA + log(QQ/mb)/(3.0*PI);}
  if ( AA<=0.0  ) {  
       cerr<<" Value of Q given was "<<QQ<<"\n";
       cerr<<" You have reached the Landau singularity.\n";
       cerr<<" All hope is lost!\n";
       exit(1);
       }
  return 1.0/AA;
}

double nflavors(double QQ) {
  double nfl;
  if ( QQ<=mb ) {nfl=4.0;}
  else {nfl=5.0;}
  return nfl;
}

