#ifndef PDFQCDHDR
#define PDFQCDHDR

/*

This gives a C++ interface to the CTEQ5 structure functions
and a uniform interface to the MRST99 NLO structure functions
August 6, 2001.  It also has a few additional functions.

Updated November 3, 2003 to include CTEQ6 structure functions
also.

*/

#include <math.h>
#include <iostream>
#include "particle.h"

enum pdf_type { cteq5 = 0, cteq = 0, mrst = 1, cteq6=2 } ;
enum collider { pp = 0, ppbar = 1 } ;

void pdfinit(pdf_type pdft, long int iset);  
           // Initialize PDF's and set appropriate alpha_s.

//  CTEQ5L (LO) : iset=3
//  CTEQ5M1 (MSbar) : iset=8
//  MRST99 central gluon:  iset=1
//  CTEQ6M (MSbar) :               iset=1
//  CTEQ6D (DIS) :                 iset=2
//  CTEQ6L (LO with NLO running) : iset=3
//  CTEQ6L1 (LO with LO running) : iset=4

double pdf(long int iparton, double x, double Q);
           // Call PDF's.

void pdfpairs(collider coll, double x1, double x2, double Q, double &fgg, 
        double &fgq, double &fqg, double &fqiqj, double &fqiqi, double &fqiai);

//  NOTE!!! pdfpairs is modified from that of pdfqcd.h.  fgq and fqg
//  are now separated.  In pdfqcd.h they were added together and called
//  simply fgq.
//  8/2/01:  Second correction.  Original pdfpairs was for ppbar.
//  Now separated into a pp and a ppbar choice.


double pdfquarks(double x, double Q);

//  Just the sum over all quark AND antiquark pdfs.

double pdfeff(double x, double Q);
           // This is the effective BFKL pdf.

double alphas(double mu);
           // Returns appropriate alpha_s, corresponding to iset for
           //   CTEQ5.

double alphasLO(double mu);
           // Should return same as alphas with iset=3.

double nflavors(double mu);

// CTEQ5 Distributions:
// ---------------------------------------------------------------------------
// Iset   PDF        Description       Alpha_s(Mz)  Lam4  Lam5   Table_File
// ---------------------------------------------------------------------------
//  1    CTEQ5M   Standard MSbar scheme   0.118     326   226    cteq5m.tbl
//  2    CTEQ5D   Standard DIS scheme     0.118     326   226    cteq5d.tbl
//  3    CTEQ5L   Leading Order           0.127     192   146    cteq5l.tbl
//  4    CTEQ5HJ  Large-x gluon enhanced  0.118     326   226    cteq5hj.tbl
//  5    CTEQ5HQ  Heavy Quark             0.118     326   226    cteq5hq.tbl
//  6    CTEQ5F3  Nf=3 FixedFlavorNumber  0.106     (Lam3=395)   cteq5f3.tbl
//  7    CTEQ5F4  Nf=4 FixedFlavorNumber  0.112     309   XXX    cteq5f4.tbl
//        --------------------------------------------------------
//  8    CTEQ5M1  Improved CTEQ5M         0.118     326   226    cteq5m1.tbl
//  9    CTEQ5HQ1 Improved CTEQ5HQ        0.118     326   226    ctq5hq1.tbl
// ---------------------------------------------------------------------------


// mrst99 class header file
// Comments regarding the C++ version to: jeppe.andersen@durham.ac.uk
// Original comments from the FORTRAN version:
//  C****************************************************************C
//  C								     C
//  C     This is a package for the new **corrected** MRST parton    C
//  C     distributions. The format is similar to the previous       C
//  C     (1998) MRST series.                                        C
//  C								     C
//  C     NOTE: 7 new sets are added here, corresponding to shifting C
//  C     the small x HERA data up and down by 2.5%, and by varying  C
//  C     the charm and strange distributions, and by forcing a      C
//  C     larger d/u ratio at large x.                               C
//  C								     C
//  C     As before, x times the parton distribution is returned,    C
//  C     q is the scale in GeV, MSbar factorization is assumed,     C
//  C     and Lambda(MSbar,nf=4) is given below for each set.        C
//  C								     C
//  C     NAMING SCHEME:                                             C
//  C						                     C
//  C  mode  set    comment             L(4)/MeV  a_s(M_Z)  grid#1   C
//  C  ----  ---    -------             --------  -------   ------   C
//  C								     C
//  C  1     COR01  central gluon, a_s    300      0.1175   0.00524  C
//  C  2     COR02  higher gluon          300      0.1175   0.00497  C
//  C  3     COR03  lower gluon           300      0.1175   0.00398  C
//  C  4     COR04  lower a_s             229      0.1125   0.00585  C
//  C  5     COR05  higher a_s            383      0.1225   0.00384  C
//  C  6     COR06  quarks up             303.3    0.1178   0.00497  C
//  C  7     COR07  quarks down           290.3    0.1171   0.00593  C
//  C  8     COR08  strange up            300      0.1175   0.00524  C
//  C  9     COR09  strange down          300      0.1175   0.00524  C
//  C  10    C0R10  charm up              300      0.1175   0.00525  C
//  C  11    COR11  charm down            300      0.1175   0.00524  C
//  C  12    COR12  larger d/u            300      0.1175   0.00515  C
//  C						                     C
//  C      The corresponding grid files are called cor01.dat etc.    C
//  C							  	     C
//  C      The reference is:                                         C
//  C      A.D. Martin, R.G. Roberts, W.J. Stirling, R.S Thorne      C
//  C      Univ. Durham preprint DTP/99/64, hep-ph/9907231 (1999)    C
//  C                                                                C
//  C      Comments to : W.J.Stirling@durham.ac.uk                   C
//  C                                                                C
//  C								     C
//  C****************************************************************C

// ---------------------------------------------------------------------------
//  Iset   PDF-set     Description       Alpha_s(Mz)**Lam4  Lam5   Table_File
// ===========================================================================
// Standard, "best-fit", sets:
// --------------------------
//   1    CTEQ6M   Standard MSbar scheme   0.118     326   226    cteq6m.tbl
//   2    CTEQ6D   Standard DIS scheme     0.118     326   226    cteq6d.tbl
//   3    CTEQ6L   Leading Order           0.118**   326** 226    cteq6l.tbl
//   4    CTEQ6L1  Leading Order           0.130**   215** 165    cteq6l1.tbl
// ============================================================================




#ifndef _MRST_H_INCLUDED_
const int np=8;
const int nx=49;
const int nq=37;
const int ntenth=23;
const double xmin=1E-5;
const double xmax=1.0;
const double qsqmin=1.25;
const double qsqmax=1E7;

struct s_partoncontent {
  double upv,dnv,usea,dsea,str,chm,bot,glu;
};

class c_mrst99function {
 private:
  double xx[nx+1];
  double qq[nq+1];
  double n0[np+1];
  double f[np+1][nx+1][nq+2];
//    double g[np+1];
 public:
  int initialise(int mode);
  struct s_partoncontent update(double x, double qsq);
};

class c_mrst {
 private:
  // Note that we start with the first function as array element zero.
  class c_mrst99function function[12]; 
 public:
  double * table[8];
  struct s_partoncontent cont;
  void mrst99(double x,double q,int mode);
  // The constructor (initialises the functions)
  c_mrst();
};
#define _MRST_H_INCLUDED_
#endif

#endif

